//heron.pde, (C) 2017 Loe Feijs and TU/e, example for Creative Programming.
//Draw a random triangle and sample it by mouseclicks.
//Test for each point whether it is in the triangle and color red or green accordingly.
//Inspiration (but not implementation): www.anderswallin.net/2007/06/point-in-triangle-test/

import processing.pdf.*;
int Ax, Ay, Bx, By, Cx, Cy;
//  coordinates of triangle points A,B and C

void setup(){
     size(500,500);
     background(255);
     fill(0);
     stroke(0);
     Ax = int(random(width));
     Ay = int(random(height));
     Bx = int(random(width));
     By = int(random(height));
     Cx = int(random(width));
     Cy = int(random(height));
     beginRecord(PDF,"HERON.pdf");
     triangle(Ax,Ay,Bx,By,Cx,Cy); 
}

void triangle(int x1, int y1, int x2, int y2, int x3, int y3){
     //draw a triangle based on points (x1,y1), (x2,y2) and (x3,y3)
     line(x1,y1,x2,y2);
     line(x2,y2,x3,y3);
     line(x3,y3,x1,y1);
}

float area(float a, float b, float c){
      //let a,b and c be the sides of a triangle
      //find the total area (using Heron's formula)
      //www.mathsisfun.com/geometry/herons-formula.html
      //en.wikipedia.org/wiki/Heron%27s_formula
      float s = (a + b + c)/2;
      float A = sqrt(s * (s-a) * (s-b) * (s-c));
      return A;
}

boolean isIn(int Px, int Py, int Ax, int Ay, int Bx, int By, int Cx, int Cy){
        //test whether point P is in the triangle with points A, B and C
        //compare area ABC with sum of areas ABP + BCP + CAP.
        //idea described on: eng-shady-mohsen.blogspot.nl/2011_09_01_archive.html
        float ABP = area(dist(Ax,Ay,Bx,By),dist(Bx,By,Px,Py),dist(Px,Py,Ax,Ay));
        float BCP = area(dist(Bx,By,Cx,Cy),dist(Cx,Cy,Px,Py),dist(Px,Py,Bx,By));
        float CAP = area(dist(Cx,Cy,Ax,Ay),dist(Ax,Ay,Px,Py),dist(Px,Py,Cx,Cy));
        return      area(dist(Ax,Ay,Bx,By),dist(Bx,By,Cx,Cy),dist(Cx,Cy,Ax,Ay))
                    >= ABP+BCP+CAP - 1;
}

void draw(){
}

void mouseClicked(){
     if (isIn(mouseX,mouseY,Ax,Ay,Bx,By,Cx,Cy))
         fill(255,0,0); //red
         else fill(0,255,0); //green
     ellipse(mouseX,mouseY,5,5);
}

void keyPressed(){
     endRecord();
     exit();
}