//heron.pde, (C) 2017 Loe Feijs and TU/e, example for Creative Programming.
//Draw a collection of MAX random triangles and sample them by random points (in draw).
//Test for each point whether it is in an odd number of triangles and draw a dark or light line-segment accordingly.
//Inspiration (but not implementation): www.anderswallin.net/2007/06/point-in-triangle-test/import processing.pdf.*;

import processing.pdf.*;

int MAX = 10;
int Ax[] = new int[MAX];
int Ay[] = new int[MAX];
int Bx[] = new int[MAX];
int By[] = new int[MAX];
int Cx[] = new int[MAX];
int Cy[] = new int[MAX];

void setup(){
     size(707,500);
     background(255);
     fill(0);
     stroke(0);
     smooth();
     for (int i = 0; i < MAX; i++){ //fill the array of coordinates
          Ax[i] = int(random(width));
          Ay[i] = int(random(height));
          Bx[i] = int(random(width));
          By[i] = int(random(height));
          Cx[i] = int(random(width));
          Cy[i] = int(random(height));
     }
     beginRecord(PDF,"HERONDESIGN.pdf");
     for (int i = 0; i < MAX; i++)//now draw them as triangles
          triangle(Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]); 
}

void triangle(int x1, int y1, int x2, int y2, int x3, int y3){
     //draw a triangle based on points (x1,y1), (x2,y2) and (x3,y3)
     line(x1,y1,x2,y2);
     line(x2,y2,x3,y3);
     line(x3,y3,x1,y1);
}

float area(float a, float b, float c){
      //let a,b and c be the sides of a triangle
      //find the total area (using Heron's formula)
      //www.mathsisfun.com/geometry/herons-formula.html
      //en.wikipedia.org/wiki/Heron%27s_formula
      float s = (a + b + c)/2;
      float A = sqrt(s * (s-a) * (s-b) * (s-c));
      return A;
}
boolean isIn(int Px, int Py, int Ax, int Ay, int Bx, int By, int Cx, int Cy){
        //test whether point P is in the triangle with points A, B and C
        //compare area ABC with sum of areas ABP + BCP + CAP.
        //idea described on: eng-shady-mohsen.blogspot.nl/2011_09_01_archive.html
        float ABP = area(dist(Ax,Ay,Bx,By),dist(Bx,By,Px,Py),dist(Px,Py,Ax,Ay));
        float BCP = area(dist(Bx,By,Cx,Cy),dist(Cx,Cy,Px,Py),dist(Px,Py,Bx,By));
        float CAP = area(dist(Cx,Cy,Ax,Ay),dist(Ax,Ay,Px,Py),dist(Px,Py,Cx,Cy));
        float ABC = area(dist(Ax,Ay,Bx,By),dist(Bx,By,Cx,Cy),dist(Cx,Cy,Ax,Ay));
        return ABC > ABP+BCP+CAP - 1;
}

void draw(){
     //test more and more points, 100 per draw
     for (int k = 0; k <100; k++){
          int x = int(random(width));
          int y = int(random(height));
          int cnt = 0;
          //counting the number of triangles (x,y) is in
          for (int i = 0; i < MAX; i++)
              if (isIn(x,y,Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]))
                  cnt++;
          noStroke();
          //choose dark colors for the even numbers, bright for odd
          if (cnt % 2 == 0)
              stroke(random(100),random(100),random(100));
              else stroke(random(150,255),random(150,255),random(150,255));
          //mark the point by either horizontal or vertical dash
          if (random(100)<50) 
              line(x-2,y,x+2,y); 
              else line(x,y-2,x,y+2);
          }
}

void keyPressed(){
     //press any key of result appears satisfactory
     endRecord();
     exit();
}